#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# PatchBay Canvas engine using QGraphicsView/Scene
# Copyright (C) 2010-2019 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the doc/GPL.txt file.

# ------------------------------------------------------------------------------------------------------------
# Imports (Global)

from PyQt5.QtCore import QAbstractAnimation
from PyQt5.QtWidgets import QGraphicsObject

# ------------------------------------------------------------------------------------------------------------
# Imports (Custom)

from . import canvas, CanvasBoxType

# ------------------------------------------------------------------------------------------------------------

class CanvasFadeAnimation(QAbstractAnimation):
    def __init__(self, item, show):
        QAbstractAnimation.__init__(self)

        self.m_show = show
        self.m_duration = 0
        self.m_item = item
        self.m_item_is_object = isinstance(item, QGraphicsObject)

    def item(self):
        return self.m_item

    def forceStop(self):
        self.blockSignals(True)
        self.stop()
        self.blockSignals(False)

    def setDuration(self, time):
        if self.m_item.opacity() == 0 and not self.m_show:
            self.m_duration = 0
        else:
            if self.m_item_is_object:
                self.m_item.blockSignals(True)
                self.m_item.show()
                self.m_item.blockSignals(False)
            else:
                self.m_item.show()
            self.m_duration = time

    def duration(self):
        return self.m_duration

    def updateCurrentTime(self, time):
        if self.m_duration == 0:
            return

        if self.m_show:
            value = float(time) / self.m_duration
        else:
            value = 1.0 - (float(time) / self.m_duration)

        try:
            self.m_item.setOpacity(value)
        except RuntimeError:
            print("CanvasFadeAnimation::updateCurrentTime() - failed to animate canvas item, already destroyed?")
            self.forceStop()
            canvas.animation_list.remove(self)
            return

        if self.m_item.type() == CanvasBoxType:
            self.m_item.setShadowOpacity(value)

    def updateDirection(self, direction):
        pass

    def updateState(self, oldState, newState):
        pass

# ------------------------------------------------------------------------------------------------------------



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# A piano roll viewer/editor
# Copyright (C) 2012-2022 Filipe Coelho <falktx@falktx.com>
# Copyright (C) 2014-2015 Perry Nguyen
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the doc/GPL.txt file.

# ------------------------------------------------------------------------------------------------------------
# Imports (Global)

from PyQt5.QtCore import Qt, QRectF, QPointF, pyqtSignal
from PyQt5.QtGui import QColor, QCursor, QFont, QPen, QPainter
from PyQt5.QtWidgets import QGraphicsItem, QGraphicsLineItem, QGraphicsRectItem, QGraphicsSimpleTextItem
from PyQt5.QtWidgets import QGraphicsScene, QGraphicsView
from PyQt5.QtWidgets import QApplication, QStyle, QWidget

# ------------------------------------------------------------------------------------------------------------
# Imports (Custom)

#from carla_shared import *

# ------------------------------------------------------------------------------------------------------------
# MIDI definitions, copied from CarlaMIDI.h

MAX_MIDI_CHANNELS = 16
MAX_MIDI_NOTE     = 128
MAX_MIDI_VALUE    = 128
MAX_MIDI_CONTROL  = 120 # 0x77

MIDI_STATUS_BIT  = 0xF0
MIDI_CHANNEL_BIT = 0x0F

# MIDI Messages List
MIDI_STATUS_NOTE_OFF              = 0x80 # note (0-127), velocity (0-127)
MIDI_STATUS_NOTE_ON               = 0x90 # note (0-127), velocity (0-127)
MIDI_STATUS_POLYPHONIC_AFTERTOUCH = 0xA0 # note (0-127), pressure (0-127)
MIDI_STATUS_CONTROL_CHANGE        = 0xB0 # see 'Control Change Messages List'
MIDI_STATUS_PROGRAM_CHANGE        = 0xC0 # program (0-127), none
MIDI_STATUS_CHANNEL_PRESSURE      = 0xD0 # pressure (0-127), none
MIDI_STATUS_PITCH_WHEEL_CONTROL   = 0xE0 # LSB (0-127), MSB (0-127)

# MIDI Message type
def MIDI_IS_CHANNEL_MESSAGE(status): return status >= MIDI_STATUS_NOTE_OFF and status <  MIDI_STATUS_BIT
def MIDI_IS_SYSTEM_MESSAGE(status):  return status >= MIDI_STATUS_BIT      and status <= 0xFF
def MIDI_IS_OSC_MESSAGE(status):     return status == '/'                   or status == '#'

# MIDI Channel message type
def MIDI_IS_STATUS_NOTE_OFF(status):
    return MIDI_IS_CHANNEL_MESSAGE(status) and (status & MIDI_STATUS_BIT) == MIDI_STATUS_NOTE_OFF
def MIDI_IS_STATUS_NOTE_ON(status):
    return MIDI_IS_CHANNEL_MESSAGE(status) and (status & MIDI_STATUS_BIT) == MIDI_STATUS_NOTE_ON
def MIDI_IS_STATUS_POLYPHONIC_AFTERTOUCH(status):
    return MIDI_IS_CHANNEL_MESSAGE(status) and (status & MIDI_STATUS_BIT) == MIDI_STATUS_POLYPHONIC_AFTERTOUCH
def MIDI_IS_STATUS_CONTROL_CHANGE(status):
    return MIDI_IS_CHANNEL_MESSAGE(status) and (status & MIDI_STATUS_BIT) == MIDI_STATUS_CONTROL_CHANGE
def MIDI_IS_STATUS_PROGRAM_CHANGE(status):
    return MIDI_IS_CHANNEL_MESSAGE(status) and (status & MIDI_STATUS_BIT) == MIDI_STATUS_PROGRAM_CHANGE
def MIDI_IS_STATUS_CHANNEL_PRESSURE(status):
    return MIDI_IS_CHANNEL_MESSAGE(status) and (status & MIDI_STATUS_BIT) == MIDI_STATUS_CHANNEL_PRESSURE
def MIDI_IS_STATUS_PITCH_WHEEL_CONTROL(status):
    return MIDI_IS_CHANNEL_MESSAGE(status) and (status & MIDI_STATUS_BIT) == MIDI_STATUS_PITCH_WHEEL_CONTROL

# MIDI Utils
def MIDI_GET_STATUS_FROM_DATA(data):
    return data[0] & MIDI_STATUS_BIT  if MIDI_IS_CHANNEL_MESSAGE(data[0]) else data[0]
def MIDI_GET_CHANNEL_FROM_DATA(data):
    return data[0] & MIDI_CHANNEL_BIT if MIDI_IS_CHANNEL_MESSAGE(data[0]) else 0

# ---------------------------------------------------------------------------------------------------------------------
# Graphics Items

class NoteExpander(QGraphicsRectItem):
    def __init__(self, length, height, parent):
        QGraphicsRectItem.__init__(self, 0, 0, length, height, parent)
        self.parent = parent
        self.orig_brush = QColor(0, 0, 0, 0)
        self.hover_brush = QColor(200, 200, 200)
        self.stretch = False

        self.setAcceptHoverEvents(True)
        self.setFlag(QGraphicsItem.ItemIsSelectable)
        self.setFlag(QGraphicsItem.ItemSendsGeometryChanges)
        self.setPen(QPen(QColor(0,0,0,0)))

    def paint(self, painter, option, widget=None):
        paint_option = option
        paint_option.state &= ~QStyle.State_Selected
        QGraphicsRectItem.paint(self, painter, paint_option, widget)

    def mousePressEvent(self, event):
        QGraphicsRectItem.mousePressEvent(self, event)
        self.stretch = True

    def mouseReleaseEvent(self, event):
        QGraphicsRectItem.mouseReleaseEvent(self, event)
        self.stretch = False

    def hoverEnterEvent(self, event):
        QGraphicsRectItem.hoverEnterEvent(self, event)
        self.setCursor(QCursor(Qt.SizeHorCursor))
        self.setBrush(self.hover_brush)

    def hoverLeaveEvent(self, event):
        QGraphicsRectItem.hoverLeaveEvent(self, event)
        self.unsetCursor()
        self.setBrush(self.orig_brush)

# ---------------------------------------------------------------------------------------------------------------------

class NoteItem(QGraphicsRectItem):
    '''a note on the pianoroll sequencer'''
    def __init__(self, height, length, note_info):
        QGraphicsRectItem.__init__(self, 0, 0, length, height)

        self.orig_brush = QColor(note_info[3], 0, 0)
        self.hover_brush = QColor(note_info[3] + 98, 200, 100)
        self.select_brush = QColor(note_info[3] + 98, 100, 100)

        self.note = note_info
        self.length = length
        self.piano = self.scene

        self.pressed = False
        self.hovering = False
        self.moving_diff = (0,0)
        self.expand_diff = 0

        self.setAcceptHoverEvents(True)
        self.setFlag(QGraphicsItem.ItemIsMovable)
        self.setFlag(QGraphicsItem.ItemIsSelectable)
        self.setFlag(QGraphicsItem.ItemSendsGeometryChanges)
        self.setPen(QPen(QColor(0,0,0,0)))
        self.setBrush(self.orig_brush)

        l = 5
        self.front = NoteExpander(l, height, self)
        self.back = NoteExpander(l, height, self)
        self.back.setPos(length - l, 0)

    def paint(self, painter, option, widget=None):
        paint_option = option
        paint_option.state &= ~QStyle.State_Selected
        if self.isSelected():
            self.setBrush(self.select_brush)
        elif self.hovering:
            self.setBrush(self.hover_brush)
        else:
            self.setBrush(self.orig_brush)
        QGraphicsRectItem.paint(self, painter, paint_option, widget)

    def hoverEnterEvent(self, event):
        QGraphicsRectItem.hoverEnterEvent(self, event)
        self.hovering = True
        self.update()
        self.setCursor(QCursor(Qt.OpenHandCursor))

    def hoverLeaveEvent(self, event):
        QGraphicsRectItem.hoverLeaveEvent(self, event)
        self.hovering = False
        self.unsetCursor()
        self.update()

    def mousePressEvent(self, event):
        QGraphicsRectItem.mousePressEvent(self, event)
        self.pressed = True
        self.moving_diff = (0,0)
        self.expand_diff = 0
        self.setCursor(QCursor(Qt.ClosedHandCursor))
        self.setSelected(True)

    def mouseMoveEvent(self, event):
        event.ignore()

    def mouseReleaseEvent(self, event):
        QGraphicsRectItem.mouseReleaseEvent(self, event)
        self.pressed = False
        self.moving_diff = (0,0)
        self.expand_diff = 0
        self.setCursor(QCursor(Qt.OpenHandCursor))

    def moveEvent(self, event):
        offset = event.scenePos() - event.lastScenePos()

        if self.back.stretch:
            self.expand(self.back, offset)
            self.updateNoteInfo(self.scenePos().x(), self.scenePos().y())
            return

        if self.front.stretch:
            self.expand(self.front, offset)
            self.updateNoteInfo(self.scenePos().x(), self.scenePos().y())
            return

        piano = self.piano()

        pos = self.scenePos() + offset + QPointF(self.moving_diff[0],self.moving_diff[1])
        pos = piano.enforce_bounds(pos)
        pos_x = pos.x()
        pos_y = pos.y()
        width = self.rect().width()
        if pos_x + width > piano.grid_width + piano.piano_width:
            pos_x = piano.grid_width + piano.piano_width - width
        pos_sx, pos_sy = piano.snap(pos_x, pos_y)

        if pos_sx + width > piano.grid_width + piano.piano_width:
            self.moving_diff = (0,0)
            self.expand_diff = 0
            return

        self.moving_diff = (pos_x-pos_sx, pos_y-pos_sy)
        self.setPos(pos_sx, pos_sy)

        self.updateNoteInfo(pos_sx, pos_sy)

    def expand(self, rectItem, offset):
        rect = self.rect()
        piano = self.piano()
        width = rect.right() + self.expand_diff

        if rectItem == self.back:
            width += offset.x()
            max_x = piano.grid_width + piano.piano_width
            if width + self.scenePos().x() >= max_x:
                width = max_x - self.scenePos().x() - 1
            elif piano.snap_value and width < piano.snap_value:
                width = piano.snap_value
            elif width < 10:
                width = 10
            new_w = piano.snap(width) - 2.75
            if new_w + self.scenePos().x() >= max_x:
                self.moving_diff = (0,0)
                self.expand_diff = 0
                return

        else:
            width -= offset.x()
            new_w = piano.snap(width+2.75) - 2.75
            if new_w <= 0:
                new_w = piano.snap_value
                self.moving_diff = (0,0)
                self.expand_diff = 0
                return
            diff = rect.right() - new_w
            if diff: # >= piano.snap_value:
                new_x = self.scenePos().x() + diff
                if new_x < piano.piano_width:
                    new_x = piano.piano_width
                    self.moving_diff = (0,0)
                    self.expand_diff = 0
                    return
                print(new_x, new_w, diff)
                self.setX(new_x)

        self.expand_diff = width - new_w
        self.back.setPos(new_w - 5, 0)
        rect.setRight(new_w)
        self.setRect(rect)

    def updateNoteInfo(self, pos_x, pos_y):
        note_info = (self.piano().get_note_num_from_y(pos_y),
                     self.piano().get_note_start_from_x(pos_x),
                     self.piano().get_note_length_from_x(self.rect().width()),
                     self.note[3])
        if self.note != note_info:
            self.piano().move_note(self.note, note_info)
            self.note = note_info

    def updateVelocity(self, event):
        offset = event.scenePos().x() - event.lastScenePos().x()
        offset = int(offset/5)

        note_info = self.note[:]
        note_info[3] += offset
        if note_info[3] > 127:
            note_info[3] = 127
        elif note_info[3] < 0:
            note_info[3] = 0
        if self.note != note_info:
            self.orig_brush = QColor(note_info[3], 0, 0)
            self.hover_brush = QColor(note_info[3] + 98, 200, 100)
            self.select_brush = QColor(note_info[3] + 98, 100, 100)
            self.update()
            self.piano().move_note(self.note, note_info)
            self.note = note_info

# ---------------------------------------------------------------------------------------------------------------------

class PianoKeyItem(QGraphicsRectItem):
    def __init__(self, width, height, note, parent):
        QGraphicsRectItem.__init__(self, 0, 0, width, height, parent)

        self.width = width
        self.height = height
        self.note = note
        self.piano = self.scene
        self.hovered = False
        self.pressed = False

        self.click_brush = QColor(255, 100, 100)
        self.hover_brush = QColor(200, 0, 0)
        self.orig_brush = None

        self.setAcceptHoverEvents(True)
        self.setFlag(QGraphicsItem.ItemIsSelectable)
        self.setPen(QPen(QColor(0,0,0,80)))

    def paint(self, painter, option, widget=None):
        paint_option = option
        paint_option.state &= ~QStyle.State_Selected
        QGraphicsRectItem.paint(self, painter, paint_option, widget)

    def hoverEnterEvent(self, event):
        QGraphicsRectItem.hoverEnterEvent(self, event)
        self.hovered = True
        self.orig_brush = self.brush()
        self.setBrush(self.hover_brush)

    def hoverLeaveEvent(self, event):
        QGraphicsRectItem.hoverLeaveEvent(self, event)
        self.hovered = False
        self.setBrush(self.click_brush if self.pressed else self.orig_brush)

    def mousePressEvent(self, event):
        QGraphicsRectItem.mousePressEvent(self, event)
        self.pressed = True
        self.setBrush(self.click_brush)
        self.piano().noteclicked.emit(self.note, True)

    def mouseReleaseEvent(self, event):
        QGraphicsRectItem.mouseReleaseEvent(self, event)
        self.pressed = False
        self.setBrush(self.hover_brush if self.hovered else self.orig_brush)
        self.piano().noteclicked.emit(self.note, False)

# ---------------------------------------------------------------------------------------------------------------------

class PianoRoll(QGraphicsScene):
    '''the piano roll'''

    noteclicked = pyqtSignal(int,bool)
    midievent = pyqtSignal(list)
    measureupdate = pyqtSignal(int)
    modeupdate = pyqtSignal(str)

    default_ghost_vel = 100

    def __init__(self, time_sig = '4/4', num_measures = 4, quantize_val = '1/8'):
        QGraphicsScene.__init__(self)
        self.setBackgroundBrush(QColor(50, 50, 50))

        self.notes = []
        self.removed_notes = []
        self.selected_notes = []
        self.piano_keys = []

        self.marquee_select = False
        self.marquee_rect = None
        self.marquee = None

        self.ghost_note = None
        self.ghost_rect = None
        self.ghost_rect_orig_width = None
        self.ghost_vel = self.default_ghost_vel

        self.ignore_mouse_events = False
        self.insert_mode = False
        self.velocity_mode = False
        self.place_ghost = False
        self.last_mouse_pos = QPointF()

        ## dimensions
        self.padding = 2

        ## piano dimensions
        self.note_height = 10
        self.start_octave = -2
        self.end_octave = 8
        self.notes_in_octave = 12
        self.total_notes = (self.end_octave - self.start_octave) * self.notes_in_octave + 1
        self.piano_height = self.note_height * self.total_notes
        self.octave_height = self.notes_in_octave * self.note_height

        self.piano_width = 34

        ## height
        self.header_height = 20
        self.total_height = self.piano_height - self.note_height + self.header_height
        #not sure why note_height is subtracted

        ## width
        self.full_note_width = 250 # i.e. a 4/4 note
        self.snap_value = None
        self.quantize_val = quantize_val

        ### dummy vars that will be changed
        self.time_sig = (0,0)
        self.measure_width = 0
        self.num_measures = 0
        self.max_note_length = 0
        self.grid_width = 0
        self.value_width = 0
        self.grid_div = 0
        self.piano = None
        self.header = None
        self.play_head = None

        self.setGridDiv()
        self.default_length = 1. / self.grid_div


    # -------------------------------------------------------------------------
    # Callbacks

    def movePlayHead(self, transportInfo):
        ticksPerBeat = transportInfo['ticksPerBeat']
        max_ticks = ticksPerBeat * self.time_sig[0] * self.num_measures
        cur_tick = ticksPerBeat * self.time_sig[0] * transportInfo['bar'] + ticksPerBeat * transportInfo['beat'] + transportInfo['tick']
        frac = (cur_tick % max_ticks) / max_ticks
        self.play_head.setPos(QPointF(frac * self.grid_width, 0))

    def setTimeSig(self, time_sig):
        self.time_sig = time_sig
        self.measure_width = self.full_note_width * self.time_sig[0]/self.time_sig[1]
        self.max_note_length = self.num_measures * self.time_sig[0]/self.time_sig[1]
        self.grid_width = self.measure_width * self.num_measures
        self.setGridDiv()

    def setMeasures(self, measures):
        #try:
        self.num_measures = float(measures)
        self.max_note_length = self.num_measures * self.time_sig[0]/self.time_sig[1]
        self.grid_width = self.measure_width * self.num_measures
        self.refreshScene()
        #except:
            #pass

    def setDefaultLength(self, length):
        v = list(map(float, length.split('/')))
        if len(v) < 3:
            self.default_length = v[0] if len(v) == 1 else v[0] / v[1]
            pos = self.enforce_bounds(self.last_mouse_pos)
            if self.insert_mode:
                self.makeGhostNote(pos.x(), pos.y())

    def setGridDiv(self, div=None):
        if not div: div = self.quantize_val
        try:
            val = list(map(int, div.split('/')))
            if len(val) < 3:
                self.quantize_val = div
                self.grid_div = val[0] if len(val)==1 else val[1]
                self.value_width = self.full_note_width / float(self.grid_div) if self.grid_div else None
                self.setQuantize(div)

                self.refreshScene()
        except ValueError:
            pass

    def setQuantize(self, value):
        val = list(map(float, value.split('/')))
        if len(val) == 1:
            self.quantize(val[0])
            self.quantize_val = value
        elif len(val) == 2:
            self.quantize(val[0] / val[1])
            self.quantize_val = value

    # -------------------------------------------------------------------------
    # Event Callbacks

    def keyPressEvent(self, event):
        QGraphicsScene.keyPressEvent(self, event)

        if event.key() == Qt.Key_Escape:
            QApplication.instance().closeAllWindows()
            return

        if event.key() == Qt.Key_F:
            if not self.insert_mode:
                # turn off velocity mode
                self.velocity_mode = False
                # enable insert mode
                self.insert_mode = True
                self.place_ghost = False
                self.makeGhostNote(self.last_mouse_pos.x(), self.last_mouse_pos.y())
                self.modeupdate.emit('insert_mode')
            else:
                # turn off insert mode
                self.insert_mode = False
                self.place_ghost = False
                if self.ghost_note is not None:
                    self.removeItem(self.ghost_note)
                    self.ghost_note = None
                self.modeupdate.emit('')

        elif event.key() == Qt.Key_D:
            if not self.velocity_mode:
                # turn off insert mode
                self.insert_mode = False
                self.place_ghost = False
                if self.ghost_note is not None:
                    self.removeItem(self.ghost_note)
                    self.ghost_note = None
                # enable velocity mode
                self.velocity_mode = True
                self.modeupdate.emit('velocity_mode')
            else:
                # turn off velocity mode
                self.velocity_mode = False
                self.modeupdate.emit('')

        elif event.key() == Qt.Key_A:
            for note in self.notes:
                if not note.isSelected():
                    has_unselected = True
                    break
            else:
                has_unselected = False

            # select all notes
            if has_unselected:
                for note in self.notes:
                    note.setSelected(True)
                self.selected_notes = self.notes[:]
            # unselect all
            else:
                for note in self.notes:
                    note.setSelected(False)
                self.selected_notes = []

        elif event.key() in (Qt.Key_Delete, Qt.Key_Backspace):
            # remove selected notes from our notes list
            self.notes = [note for note in self.notes if note not in self.selected_notes]
            # delete the selected notes
            for note in self.selected_notes:
                self.removeItem(note)
                self.midievent.emit(["midievent-remove", note.note[0], note.note[1], note.note[2], note.note[3]])
                del note
            self.selected_notes = []

    def mousePressEvent(self, event):
        QGraphicsScene.mousePressEvent(self, event)

        # mouse click on left-side piano area
        if self.piano.contains(event.scenePos()):
            self.ignore_mouse_events = True
            return

        clicked_notes = []

        for note in self.notes:
            if note.pressed or note.back.stretch or note.front.stretch:
                clicked_notes.append(note)

        # mouse click on existing notes
        if clicked_notes:
            keep_selection = all(note in self.selected_notes for note in clicked_notes)
            if keep_selection:
                for note in self.selected_notes:
                    note.setSelected(True)
                return

            for note in self.selected_notes:
                if note not in clicked_notes:
                    note.setSelected(False)
            for note in clicked_notes:
                if note not in self.selected_notes:
                    note.setSelected(True)

            self.selected_notes = clicked_notes
            return

        # mouse click on empty area (no note selected)
        for note in self.selected_notes:
            note.setSelected(False)
        self.selected_notes = []

        if event.button() != Qt.LeftButton:
            return

        if self.insert_mode:
            self.place_ghost = True
        else:
            self.marquee_select = True
            self.marquee_rect = QRectF(event.scenePos().x(), event.scenePos().y(), 1, 1)
            self.marquee = QGraphicsRectItem(self.marquee_rect)
            self.marquee.setBrush(QColor(255, 255, 255, 100))
            self.addItem(self.marquee)

    def mouseMoveEvent(self, event):
        QGraphicsScene.mouseMoveEvent(self, event)

        self.last_mouse_pos = event.scenePos()

        if self.ignore_mouse_events:
            return

        pos = self.enforce_bounds(self.last_mouse_pos)

        if self.insert_mode:
            if self.ghost_note is None:
                self.makeGhostNote(pos.x(), pos.y())
            max_x = self.grid_width + self.piano_width

            # placing note, only width needs updating
            if self.place_ghost:
                pos_x = pos.x()
                min_x = self.ghost_rect.x() + self.ghost_rect_orig_width
                if pos_x < min_x:
                    pos_x = min_x
                new_x = self.snap(pos_x)
                self.ghost_rect.setRight(new_x)
                self.ghost_note.setRect(self.ghost_rect)
                #self.adjust_note_vel(event)

            # ghostnote following mouse around
            else:
                pos_x = pos.x()
                if pos_x + self.ghost_rect.width() >= max_x:
                    pos_x = max_x - self.ghost_rect.width()
                elif pos_x > self.piano_width + self.ghost_rect.width()*3/4:
                    pos_x -= self.ghost_rect.width()/2
                new_x, new_y = self.snap(pos_x, pos.y())
                self.ghost_rect.moveTo(new_x, new_y)
                self.ghost_note.setRect(self.ghost_rect)
            return

        if self.marquee_select:
            marquee_orig_pos = event.buttonDownScenePos(Qt.LeftButton)
            if marquee_orig_pos.x() < pos.x() and marquee_orig_pos.y() < pos.y():
                self.marquee_rect.setBottomRight(pos)
            elif marquee_orig_pos.x() < pos.x() and marquee_orig_pos.y() > pos.y():
                self.marquee_rect.setTopRight(pos)
            elif marquee_orig_pos.x() > pos.x() and marquee_orig_pos.y() < pos.y():
                self.marquee_rect.setBottomLeft(pos)
            elif marquee_orig_pos.x() > pos.x() and marquee_orig_pos.y() > pos.y():
                self.marquee_rect.setTopLeft(pos)
            self.marquee.setRect(self.marquee_rect)

            for note in self.selected_notes:
                note.setSelected(False)
            self.selected_notes = []

            for item in self.collidingItems(self.marquee):
                if item in self.notes:
                    item.setSelected(True)
                    self.selected_notes.append(item)
            return

        if event.buttons() != Qt.LeftButton:
            return

        if self.velocity_mode:
            for note in self.selected_notes:
                note.updateVelocity(event)
            return

        x = y = False
        for note in self.selected_notes:
            if note.back.stretch:
                x = True
                break
        for note in self.selected_notes:
            if note.front.stretch:
                y = True
                break
        for note in self.selected_notes:
            note.back.stretch = x
            note.front.stretch = y
            note.moveEvent(event)

    def mouseReleaseEvent(self, event):
        QGraphicsScene.mouseReleaseEvent(self, event)

        if self.ignore_mouse_events:
            self.ignore_mouse_events = False
            return

        if self.marquee_select:
            self.marquee_select = False
            self.removeItem(self.marquee)
            self.marquee = None

        if self.insert_mode and self.place_ghost:
            self.place_ghost = False
            note_start = self.get_note_start_from_x(self.ghost_rect.x())
            note_num = self.get_note_num_from_y(self.ghost_rect.y())
            note_length = self.get_note_length_from_x(self.ghost_rect.width())
            note = self.drawNote(note_num, note_start, note_length, self.ghost_vel)
            note.setSelected(True)
            self.selected_notes.append(note)
            self.midievent.emit(["midievent-add", note_num, note_start, note_length, self.ghost_vel])
            pos = self.enforce_bounds(self.last_mouse_pos)
            pos_x = pos.x()
            if pos_x > self.piano_width + self.ghost_rect.width()*3/4:
                pos_x -= self.ghost_rect.width()/2
            self.makeGhostNote(pos_x, pos.y())

        for note in self.selected_notes:
            note.back.stretch = False
            note.front.stretch = False

    # -------------------------------------------------------------------------
    # Internal Functions

    def drawHeader(self):
        self.header = QGraphicsRectItem(0, 0, self.grid_width, self.header_height)
        #self.header.setZValue(1.0)
        self.header.setPos(self.piano_width, 0)
        self.addItem(self.header)

    def drawPiano(self):
        piano_keys_width = self.piano_width - self.padding
        labels = ('B','Bb','A','Ab','G','Gb','F','E','Eb','D','Db','C')
        black_notes = (2,4,6,9,11)
        piano_label = QFont()
        piano_label.setPointSize(6)
        self.piano = QGraphicsRectItem(0, 0, piano_keys_width, self.piano_height)
        self.piano.setPos(0, self.header_height)
        self.addItem(self.piano)

        key = PianoKeyItem(piano_keys_width, self.note_height, 78, self.piano)
        label = QGraphicsSimpleTextItem('C9', key)
        label.setPos(18, 1)
        label.setFont(piano_label)
        key.setBrush(QColor(255, 255, 255))
        for i in range(self.end_octave - self.start_octave, 0, -1):
            for j in range(self.notes_in_octave, 0, -1):
                note = (self.end_octave - i + 3) * 12 - j
                if j in black_notes:
                    key = PianoKeyItem(piano_keys_width/1.4, self.note_height, note, self.piano)
                    key.setBrush(QColor(0, 0, 0))
                    key.setZValue(1.0)
                    key.setPos(0, self.note_height * j + self.octave_height * (i - 1))
                elif (j - 1) and (j + 1) in black_notes:
                    key = PianoKeyItem(piano_keys_width, self.note_height * 2, note, self.piano)
                    key.setBrush(QColor(255, 255, 255))
                    key.setPos(0, self.note_height * j + self.octave_height * (i - 1) - self.note_height/2.)
                elif (j - 1) in black_notes:
                    key = PianoKeyItem(piano_keys_width, self.note_height * 3./2, note, self.piano)
                    key.setBrush(QColor(255, 255, 255))
                    key.setPos(0, self.note_height * j + self.octave_height * (i - 1) - self.note_height/2.)
                elif (j + 1) in black_notes:
                    key = PianoKeyItem(piano_keys_width, self.note_height * 3./2, note, self.piano)
                    key.setBrush(QColor(255, 255, 255))
                    key.setPos(0, self.note_height * j + self.octave_height * (i - 1))
                if j == 12:
                    label = QGraphicsSimpleTextItem('{}{}'.format(labels[j - 1], self.end_octave - i + 1), key)
                    label.setPos(18, 6)
                    label.setFont(piano_label)
                self.piano_keys.append(key)

    def drawGrid(self):
        black_notes = [2,4,6,9,11]
        scale_bar = QGraphicsRectItem(0, 0, self.grid_width, self.note_height, self.piano)
        scale_bar.setPos(self.piano_width, 0)
        scale_bar.setBrush(QColor(100,100,100))
        clearpen = QPen(QColor(0,0,0,0))
        for i in range(self.end_octave - self.start_octave, self.start_octave - self.start_octave, -1):
            for j in range(self.notes_in_octave, 0, -1):
                scale_bar = QGraphicsRectItem(0, 0, self.grid_width, self.note_height, self.piano)
                scale_bar.setPos(self.piano_width, self.note_height * j + self.octave_height * (i - 1))
                scale_bar.setPen(clearpen)
                if j not in black_notes:
                    scale_bar.setBrush(QColor(120,120,120))
                else:
                    scale_bar.setBrush(QColor(100,100,100))

        measure_pen = QPen(QColor(0, 0, 0, 120), 3)
        half_measure_pen = QPen(QColor(0, 0, 0, 40), 2)
        line_pen = QPen(QColor(0, 0, 0, 40))
        for i in range(0, int(self.num_measures) + 1):
            measure = QGraphicsLineItem(0, 0, 0, self.piano_height + self.header_height - measure_pen.width(), self.header)
            measure.setPos(self.measure_width * i, 0.5 * measure_pen.width())
            measure.setPen(measure_pen)
            if i < self.num_measures:
                number = QGraphicsSimpleTextItem('%d' % (i + 1), self.header)
                number.setPos(self.measure_width * i + 5, 2)
                number.setBrush(Qt.white)
                for j in self.frange(0, self.time_sig[0]*self.grid_div/self.time_sig[1], 1.):
                    line = QGraphicsLineItem(0, 0, 0, self.piano_height, self.header)
                    line.setZValue(1.0)
                    line.setPos(self.measure_width * i + self.value_width * j, self.header_height)
                    if j == self.time_sig[0]*self.grid_div/self.time_sig[1] / 2.0:
                        line.setPen(half_measure_pen)
                    else:
                        line.setPen(line_pen)

    def drawPlayHead(self):
        self.play_head = QGraphicsLineItem(self.piano_width, self.header_height, self.piano_width, self.total_height)
        self.play_head.setPen(QPen(QColor(255,255,255,50), 2))
        self.play_head.setZValue(1.)
        self.addItem(self.play_head)

    def refreshScene(self):
        list(map(self.removeItem, self.notes))
        self.selected_notes = []
        self.piano_keys = []
        self.place_ghost = False
        if self.ghost_note is not None:
            self.removeItem(self.ghost_note)
            self.ghost_note = None
        self.clear()
        self.drawPiano()
        self.drawHeader()
        self.drawGrid()
        self.drawPlayHead()

        for note in self.notes[:]:
            if note.note[1] >= (self.num_measures * self.time_sig[0]):
                self.notes.remove(note)
                self.removed_notes.append(note)
                #self.midievent.emit(["midievent-remove", note.note[0], note.note[1], note.note[2], note.note[3]])
            elif note.note[2] > self.max_note_length:
                new_note = note.note[:]
                new_note[2] = self.max_note_length
                self.notes.remove(note)
                self.drawNote(new_note[0], new_note[1], self.max_note_length, new_note[3], False)
                self.midievent.emit(["midievent-remove", note.note[0], note.note[1], note.note[2], note.note[3]])
                self.midievent.emit(["midievent-add", new_note[0], new_note[1], new_note[2], new_note[3]])

        for note in self.removed_notes[:]:
            if note.note[1] < (self.num_measures * self.time_sig[0]):
                self.removed_notes.remove(note)
                self.notes.append(note)

        list(map(self.addItem, self.notes))
        if self.views():
            self.views()[0].setSceneRect(self.itemsBoundingRect())

    def clearNotes(self):
        self.clear()
        self.notes = []
        self.removed_notes = []
        self.selected_notes = []
        self.drawPiano()
        self.drawHeader()
        self.drawGrid()

    def makeGhostNote(self, pos_x, pos_y):
        """creates the ghostnote that is placed on the scene before the real one is."""
        if self.ghost_note is not None:
            self.removeItem(self.ghost_note)
        length = self.full_note_width * self.default_length
        pos_x, pos_y = self.snap(pos_x, pos_y)
        self.ghost_vel = self.default_ghost_vel
        self.ghost_rect = QRectF(pos_x, pos_y, length, self.note_height)
        self.ghost_rect_orig_width = self.ghost_rect.width()
        self.ghost_note = QGraphicsRectItem(self.ghost_rect)
        self.ghost_note.setBrush(QColor(230, 221, 45, 100))
        self.addItem(self.ghost_note)

    def drawNote(self, note_num, note_start, note_length, note_velocity, add=True):
        """
        note_num: midi number, 0 - 127
        note_start: 0 - (num_measures * time_sig[0]) so this is in beats
        note_length: 0 - (num_measures  * time_sig[0]/time_sig[1]) this is in measures
        note_velocity: 0 - 127
        """

        info = [note_num, note_start, note_length, note_velocity]

        if not note_start % (self.num_measures * self.time_sig[0]) == note_start:
            #self.midievent.emit(["midievent-remove", note_num, note_start, note_length, note_velocity])
            while not note_start % (self.num_measures * self.time_sig[0]) == note_start:
                self.setMeasures(self.num_measures+1)
            self.measureupdate.emit(self.num_measures)
            self.refreshScene()

        x_start = self.get_note_x_start(note_start)
        if note_length > self.max_note_length:
            note_length = self.max_note_length + 0.25
        x_length = self.get_note_x_length(note_length)
        y_pos = self.get_note_y_pos(note_num)

        note = NoteItem(self.note_height, x_length, info)
        note.setPos(x_start, y_pos)

        self.notes.append(note)

        if add:
            self.addItem(note)

        return note

    # -------------------------------------------------------------------------
    # Helper Functions

    def frange(self, x, y, t):
        while x < y:
            yield x
            x += t

    def quantize(self, value):
        self.snap_value = float(self.full_note_width) * value if value else None

    def snap(self, pos_x, pos_y = None):
        if self.snap_value:
            pos_x = int(round((pos_x - self.piano_width) / self.snap_value)) * self.snap_value + self.piano_width
        if pos_y is not None:
            pos_y = int((pos_y - self.header_height) / self.note_height) * self.note_height + self.header_height
        return (pos_x, pos_y) if pos_y is not None else pos_x

    def adjust_note_vel(self, event):
        m_pos = event.scenePos()
        #bind velocity to vertical mouse movement
        self.ghost_vel += (event.lastScenePos().y() - m_pos.y())/10
        if self.ghost_vel < 0:
            self.ghost_vel = 0
        elif self.ghost_vel > 127:
            self.ghost_vel = 127

        m_width = self.ghost_rect.x() + self.ghost_rect_orig_width
        if m_pos.x() < m_width:
            m_pos.setX(m_width)
        m_new_x = self.snap(m_pos.x())
        self.ghost_rect.setRight(m_new_x)
        self.ghost_note.setRect(self.ghost_rect)

    def enforce_bounds(self, pos):
        pos = QPointF(pos)
        if pos.x() < self.piano_width:
            pos.setX(self.piano_width)
        elif pos.x() >= self.grid_width + self.piano_width:
            pos.setX(self.grid_width + self.piano_width - 1)
        if pos.y() < self.header_height + self.padding:
            pos.setY(self.header_height + self.padding)
        return pos

    def get_note_start_from_x(self, note_x):
        return (note_x - self.piano_width) / (self.grid_width / self.num_measures / self.time_sig[0])

    def get_note_x_start(self, note_start):
        return self.piano_width + (self.grid_width / self.num_measures / self.time_sig[0]) * note_start

    def get_note_x_length(self, note_length):
        return float(self.time_sig[1]) / self.time_sig[0] * note_length * self.grid_width / self.num_measures

    def get_note_length_from_x(self, note_x):
        return float(self.time_sig[0]) / self.time_sig[1] * self.num_measures / self.grid_width * note_x

    def get_note_y_pos(self, note_num):
        return self.header_height + self.note_height * (self.total_notes - note_num - 1)

    def get_note_num_from_y(self, note_y_pos):
        return -(int((note_y_pos - self.header_height) / self.note_height) - self.total_notes + 1)

    def move_note(self, old_note, new_note):
        self.midievent.emit(["midievent-remove", old_note[0], old_note[1], old_note[2], old_note[3]])
        self.midievent.emit(["midievent-add", new_note[0], new_note[1], new_note[2], new_note[3]])

# ------------------------------------------------------------------------------------------------------------

class PianoRollView(QGraphicsView):
    def __init__(self, parent, time_sig = '4/4', num_measures = 4, quantize_val = '1/8'):
        QGraphicsView.__init__(self, parent)
        self.piano = PianoRoll(time_sig, num_measures, quantize_val)
        self.setScene(self.piano)
        #self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        x = 0   * self.sceneRect().width() + self.sceneRect().left()
        y = 0.4 * self.sceneRect().height() + self.sceneRect().top()
        self.centerOn(x, y)

        self.setAlignment(Qt.AlignLeft)
        self.o_transform = self.transform()
        self.zoom_x = 1
        self.zoom_y = 1

    def setZoomX(self, scale_x):
        self.setTransform(self.o_transform)
        self.zoom_x = 1 + scale_x / float(99) * 2
        self.scale(self.zoom_x, self.zoom_y)

    def setZoomY(self, scale_y):
        self.setTransform(self.o_transform)
        self.zoom_y = 1 + scale_y / float(99)
        self.scale(self.zoom_x, self.zoom_y)

# ------------------------------------------------------------------------------------------------------------

class ModeIndicator(QWidget):
    def __init__(self, parent):
        QWidget.__init__(self, parent)
        #self.setGeometry(0, 0, 30, 20)
        self.mode = None
        self.setFixedSize(30,20)

    def paintEvent(self, event):
        event.accept()

        painter = QPainter(self)
        painter.setPen(QPen(QColor(0, 0, 0, 0)))

        if self.mode == 'velocity_mode':
            painter.setBrush(QColor(127, 0, 0))
        elif self.mode == 'insert_mode':
            painter.setBrush(QColor(0, 100, 127))
        else:
            painter.setBrush(QColor(0, 0, 0, 0))

        painter.drawRect(0, 0, 30, 20)

    def changeMode(self, new_mode):
        self.mode = new_mode
        self.update()

# ------------------------------------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_host.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CarlaHostW(object):
    def setupUi(self, CarlaHostW):
        CarlaHostW.setObjectName("CarlaHostW")
        CarlaHostW.resize(1045, 741)
        self.centralwidget = QtWidgets.QWidget(CarlaHostW)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_4.setContentsMargins(0, 4, 0, 1)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.tabWidget.setObjectName("tabWidget")
        self.rack = QtWidgets.QWidget()
        self.rack.setObjectName("rack")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.rack)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 1)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.frame = QtWidgets.QFrame(self.rack)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.frame.setObjectName("frame")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout(self.frame)
        self.horizontalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_10.setSpacing(0)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.pad_left = QtWidgets.QLabel(self.frame)
        self.pad_left.setMinimumSize(QtCore.QSize(25, 0))
        self.pad_left.setMaximumSize(QtCore.QSize(25, 16777215))
        self.pad_left.setText("")
        self.pad_left.setObjectName("pad_left")
        self.horizontalLayout_10.addWidget(self.pad_left)
        self.listWidget = RackListWidget(self.frame)
        self.listWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.listWidget.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.listWidget.setFrameShadow(QtWidgets.QFrame.Plain)
        self.listWidget.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.listWidget.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.listWidget.setObjectName("listWidget")
        self.horizontalLayout_10.addWidget(self.listWidget)
        self.pad_right = QtWidgets.QLabel(self.frame)
        self.pad_right.setMinimumSize(QtCore.QSize(25, 0))
        self.pad_right.setMaximumSize(QtCore.QSize(25, 16777215))
        self.pad_right.setText("")
        self.pad_right.setObjectName("pad_right")
        self.horizontalLayout_10.addWidget(self.pad_right)
        self.rackScrollBar = QtWidgets.QScrollBar(self.frame)
        self.rackScrollBar.setOrientation(QtCore.Qt.Vertical)
        self.rackScrollBar.setObjectName("rackScrollBar")
        self.horizontalLayout_10.addWidget(self.rackScrollBar)
        self.horizontalLayout_2.addWidget(self.frame)
        self.tabWidget.addTab(self.rack, "")
        self.patchbay = QtWidgets.QWidget()
        self.patchbay.setObjectName("patchbay")
        self.gridLayout = QtWidgets.QGridLayout(self.patchbay)
        self.gridLayout.setContentsMargins(0, 0, 0, 1)
        self.gridLayout.setSpacing(1)
        self.gridLayout.setObjectName("gridLayout")
        self.graphicsView = DraggableGraphicsView(self.patchbay)
        self.graphicsView.setTransformationAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)
        self.graphicsView.setObjectName("graphicsView")
        self.gridLayout.addWidget(self.graphicsView, 0, 1, 1, 1)
        self.peak_out = DigitalPeakMeter(self.patchbay)
        self.peak_out.setObjectName("peak_out")
        self.gridLayout.addWidget(self.peak_out, 0, 2, 1, 1)
        self.peak_in = DigitalPeakMeter(self.patchbay)
        self.peak_in.setObjectName("peak_in")
        self.gridLayout.addWidget(self.peak_in, 0, 0, 1, 1)
        self.tabWidget.addTab(self.patchbay, "")
        self.logs = QtWidgets.QWidget()
        self.logs.setObjectName("logs")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.logs)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 1)
        self.horizontalLayout_4.setSpacing(1)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.text_logs = QtWidgets.QPlainTextEdit(self.logs)
        font = QtGui.QFont()
        font.setFamily("DejaVu Sans Mono")
        self.text_logs.setFont(font)
        self.text_logs.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.text_logs.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.text_logs.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.text_logs.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.text_logs.setObjectName("text_logs")
        self.horizontalLayout_4.addWidget(self.text_logs)
        self.log_options = QtWidgets.QWidget(self.logs)
        self.log_options.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.log_options.sizePolicy().hasHeightForWidth())
        self.log_options.setSizePolicy(sizePolicy)
        self.log_options.setMinimumSize(QtCore.QSize(0, 0))
        self.log_options.setObjectName("log_options")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.log_options)
        self.verticalLayout_8.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_8.setContentsMargins(10, 12, 10, -1)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.logs_save = QtWidgets.QPushButton(self.log_options)
        self.logs_save.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.logs_save.setAcceptDrops(False)
        self.logs_save.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.logs_save.setAutoFillBackground(False)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/document-save.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.logs_save.setIcon(icon)
        self.logs_save.setIconSize(QtCore.QSize(16, 16))
        self.logs_save.setAutoDefault(False)
        self.logs_save.setDefault(False)
        self.logs_save.setFlat(False)
        self.logs_save.setObjectName("logs_save")
        self.verticalLayout_8.addWidget(self.logs_save)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem)
        self.logs_clear = QtWidgets.QPushButton(self.log_options)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/16x16/edit-clear.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.logs_clear.setIcon(icon1)
        self.logs_clear.setIconSize(QtCore.QSize(16, 16))
        self.logs_clear.setObjectName("logs_clear")
        self.verticalLayout_8.addWidget(self.logs_clear)
        spacerItem1 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem1)
        self.logs_autoscroll = QtWidgets.QCheckBox(self.log_options)
        self.logs_autoscroll.setChecked(True)
        self.logs_autoscroll.setObjectName("logs_autoscroll")
        self.verticalLayout_8.addWidget(self.logs_autoscroll)
        self.horizontalLayout_4.addWidget(self.log_options, 0, QtCore.Qt.AlignTop)
        self.tabWidget.addTab(self.logs, "")
        self.verticalLayout_4.addWidget(self.tabWidget)
        self.tw_statusbar = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tw_statusbar.sizePolicy().hasHeightForWidth())
        self.tw_statusbar.setSizePolicy(sizePolicy)
        self.tw_statusbar.setObjectName("tw_statusbar")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tw_statusbar)
        self.horizontalLayout_3.setContentsMargins(6, 0, 6, 4)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_buffer_size = QtWidgets.QLabel(self.tw_statusbar)
        self.label_buffer_size.setObjectName("label_buffer_size")
        self.horizontalLayout_3.addWidget(self.label_buffer_size)
        self.cb_buffer_size = QtWidgets.QComboBox(self.tw_statusbar)
        self.cb_buffer_size.setObjectName("cb_buffer_size")
        self.horizontalLayout_3.addWidget(self.cb_buffer_size)
        self.line_1 = QtWidgets.QFrame(self.tw_statusbar)
        self.line_1.setLineWidth(0)
        self.line_1.setMidLineWidth(1)
        self.line_1.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_1.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_1.setObjectName("line_1")
        self.horizontalLayout_3.addWidget(self.line_1)
        self.label_sample_rate = QtWidgets.QLabel(self.tw_statusbar)
        self.label_sample_rate.setObjectName("label_sample_rate")
        self.horizontalLayout_3.addWidget(self.label_sample_rate)
        self.cb_sample_rate = QtWidgets.QComboBox(self.tw_statusbar)
        self.cb_sample_rate.setObjectName("cb_sample_rate")
        self.horizontalLayout_3.addWidget(self.cb_sample_rate)
        self.line_2 = QtWidgets.QFrame(self.tw_statusbar)
        self.line_2.setLineWidth(0)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_3.addWidget(self.line_2)
        self.b_xruns = QtWidgets.QPushButton(self.tw_statusbar)
        self.b_xruns.setFlat(True)
        self.b_xruns.setObjectName("b_xruns")
        self.horizontalLayout_3.addWidget(self.b_xruns)
        self.line_4 = QtWidgets.QFrame(self.tw_statusbar)
        self.line_4.setLineWidth(0)
        self.line_4.setMidLineWidth(1)
        self.line_4.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.horizontalLayout_3.addWidget(self.line_4)
        self.pb_dsp_load = QtWidgets.QProgressBar(self.tw_statusbar)
        self.pb_dsp_load.setProperty("value", 0)
        self.pb_dsp_load.setAlignment(QtCore.Qt.AlignCenter)
        self.pb_dsp_load.setObjectName("pb_dsp_load")
        self.horizontalLayout_3.addWidget(self.pb_dsp_load)
        self.verticalLayout_4.addWidget(self.tw_statusbar)
        CarlaHostW.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(CarlaHostW)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1045, 25))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Engine = QtWidgets.QMenu(self.menubar)
        self.menu_Engine.setObjectName("menu_Engine")
        self.menu_Plugin = QtWidgets.QMenu(self.menubar)
        self.menu_Plugin.setObjectName("menu_Plugin")
        self.menu_PluginMacros = QtWidgets.QMenu(self.menu_Plugin)
        self.menu_PluginMacros.setObjectName("menu_PluginMacros")
        self.menu_Canvas = QtWidgets.QMenu(self.menubar)
        self.menu_Canvas.setObjectName("menu_Canvas")
        self.menu_Canvas_Zoom = QtWidgets.QMenu(self.menu_Canvas)
        self.menu_Canvas_Zoom.setObjectName("menu_Canvas_Zoom")
        self.menu_Settings = QtWidgets.QMenu(self.menubar)
        self.menu_Settings.setObjectName("menu_Settings")
        self.menu_Help = QtWidgets.QMenu(self.menubar)
        self.menu_Help.setObjectName("menu_Help")
        CarlaHostW.setMenuBar(self.menubar)
        self.toolBar = QtWidgets.QToolBar(CarlaHostW)
        self.toolBar.setMovable(False)
        self.toolBar.setAllowedAreas(QtCore.Qt.NoToolBarArea)
        self.toolBar.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.toolBar.setObjectName("toolBar")
        CarlaHostW.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.dockWidget = QtWidgets.QDockWidget(CarlaHostW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(10)
        sizePolicy.setHeightForWidth(self.dockWidget.sizePolicy().hasHeightForWidth())
        self.dockWidget.setSizePolicy(sizePolicy)
        self.dockWidget.setFeatures(QtWidgets.QDockWidget.NoDockWidgetFeatures)
        self.dockWidget.setAllowedAreas(QtCore.Qt.NoDockWidgetArea)
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setContentsMargins(1, 5, 0, 1)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(self.dockWidgetContents)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.tabUtils = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(10)
        sizePolicy.setHeightForWidth(self.tabUtils.sizePolicy().hasHeightForWidth())
        self.tabUtils.setSizePolicy(sizePolicy)
        self.tabUtils.setMinimumSize(QtCore.QSize(210, 0))
        self.tabUtils.setAcceptDrops(False)
        self.tabUtils.setTabPosition(QtWidgets.QTabWidget.West)
        self.tabUtils.setObjectName("tabUtils")
        self.disk = QtWidgets.QWidget()
        self.disk.setAcceptDrops(False)
        self.disk.setObjectName("disk")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.disk)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 1)
        self.verticalLayout_3.setSpacing(1)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.cb_disk = QtWidgets.QComboBox(self.disk)
        self.cb_disk.setObjectName("cb_disk")
        self.cb_disk.addItem("")
        self.horizontalLayout.addWidget(self.cb_disk)
        self.b_disk_add = QtWidgets.QToolButton(self.disk)
        self.b_disk_add.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/16x16/list-add.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_disk_add.setIcon(icon2)
        self.b_disk_add.setObjectName("b_disk_add")
        self.horizontalLayout.addWidget(self.b_disk_add)
        self.b_disk_remove = QtWidgets.QToolButton(self.disk)
        self.b_disk_remove.setEnabled(False)
        self.b_disk_remove.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/16x16/list-remove.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_disk_remove.setIcon(icon3)
        self.b_disk_remove.setObjectName("b_disk_remove")
        self.horizontalLayout.addWidget(self.b_disk_remove)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.fileTreeView = QtWidgets.QTreeView(self.disk)
        self.fileTreeView.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.fileTreeView.setDragEnabled(True)
        self.fileTreeView.setDragDropMode(QtWidgets.QAbstractItemView.DragOnly)
        self.fileTreeView.setObjectName("fileTreeView")
        self.verticalLayout_3.addWidget(self.fileTreeView)
        self.tabUtils.addTab(self.disk, "")
        self.w_transport = QtWidgets.QWidget()
        self.w_transport.setObjectName("w_transport")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.w_transport)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.group_transport_controls = QtWidgets.QGroupBox(self.w_transport)
        self.group_transport_controls.setObjectName("group_transport_controls")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.group_transport_controls)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.b_transport_play = QtWidgets.QPushButton(self.group_transport_controls)
        self.b_transport_play.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/16x16/media-playback-start.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_transport_play.setIcon(icon4)
        self.b_transport_play.setCheckable(True)
        self.b_transport_play.setObjectName("b_transport_play")
        self.horizontalLayout_5.addWidget(self.b_transport_play)
        self.b_transport_stop = QtWidgets.QPushButton(self.group_transport_controls)
        self.b_transport_stop.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/16x16/media-playback-stop.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_transport_stop.setIcon(icon5)
        self.b_transport_stop.setObjectName("b_transport_stop")
        self.horizontalLayout_5.addWidget(self.b_transport_stop)
        self.b_transport_backwards = QtWidgets.QPushButton(self.group_transport_controls)
        self.b_transport_backwards.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/16x16/media-seek-backward.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_transport_backwards.setIcon(icon6)
        self.b_transport_backwards.setAutoRepeat(True)
        self.b_transport_backwards.setObjectName("b_transport_backwards")
        self.horizontalLayout_5.addWidget(self.b_transport_backwards)
        self.b_transport_forwards = QtWidgets.QPushButton(self.group_transport_controls)
        self.b_transport_forwards.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/16x16/media-seek-forward.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_transport_forwards.setIcon(icon7)
        self.b_transport_forwards.setAutoRepeat(True)
        self.b_transport_forwards.setObjectName("b_transport_forwards")
        self.horizontalLayout_5.addWidget(self.b_transport_forwards)
        self.verticalLayout_6.addWidget(self.group_transport_controls)
        self.group_transport_info = QtWidgets.QGroupBox(self.w_transport)
        self.group_transport_info.setObjectName("group_transport_info")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.group_transport_info)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_transport_frame = QtWidgets.QLabel(self.group_transport_info)
        self.label_transport_frame.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_transport_frame.setObjectName("label_transport_frame")
        self.horizontalLayout_6.addWidget(self.label_transport_frame)
        self.l_transport_frame = QtWidgets.QLabel(self.group_transport_info)
        font = QtGui.QFont()
        font.setFamily("DejaVu Sans Mono")
        self.l_transport_frame.setFont(font)
        self.l_transport_frame.setObjectName("l_transport_frame")
        self.horizontalLayout_6.addWidget(self.l_transport_frame)
        spacerItem2 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_transport_time = QtWidgets.QLabel(self.group_transport_info)
        self.label_transport_time.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_transport_time.setObjectName("label_transport_time")
        self.horizontalLayout_7.addWidget(self.label_transport_time)
        self.l_transport_time = QtWidgets.QLabel(self.group_transport_info)
        font = QtGui.QFont()
        font.setFamily("DejaVu Sans Mono")
        self.l_transport_time.setFont(font)
        self.l_transport_time.setObjectName("l_transport_time")
        self.horizontalLayout_7.addWidget(self.l_transport_time)
        spacerItem3 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_transport_bbt = QtWidgets.QLabel(self.group_transport_info)
        self.label_transport_bbt.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_transport_bbt.setObjectName("label_transport_bbt")
        self.horizontalLayout_8.addWidget(self.label_transport_bbt)
        self.l_transport_bbt = QtWidgets.QLabel(self.group_transport_info)
        font = QtGui.QFont()
        font.setFamily("DejaVu Sans Mono")
        self.l_transport_bbt.setFont(font)
        self.l_transport_bbt.setObjectName("l_transport_bbt")
        self.horizontalLayout_8.addWidget(self.l_transport_bbt)
        spacerItem4 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        self.verticalLayout_6.addWidget(self.group_transport_info)
        self.group_transport_settings = QtWidgets.QGroupBox(self.w_transport)
        self.group_transport_settings.setObjectName("group_transport_settings")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.group_transport_settings)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.dsb_transport_bpm = QtWidgets.QDoubleSpinBox(self.group_transport_settings)
        self.dsb_transport_bpm.setDecimals(1)
        self.dsb_transport_bpm.setMinimum(20.0)
        self.dsb_transport_bpm.setMaximum(999.0)
        self.dsb_transport_bpm.setObjectName("dsb_transport_bpm")
        self.verticalLayout_5.addWidget(self.dsb_transport_bpm)
        self.cb_transport_jack = QtWidgets.QCheckBox(self.group_transport_settings)
        self.cb_transport_jack.setObjectName("cb_transport_jack")
        self.verticalLayout_5.addWidget(self.cb_transport_jack)
        self.cb_transport_link = QtWidgets.QCheckBox(self.group_transport_settings)
        self.cb_transport_link.setObjectName("cb_transport_link")
        self.verticalLayout_5.addWidget(self.cb_transport_link)
        self.verticalLayout_6.addWidget(self.group_transport_settings)
        spacerItem5 = QtWidgets.QSpacerItem(20, 188, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem5)
        self.tabUtils.addTab(self.w_transport, "")
        self.tw_miniCanvas = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tw_miniCanvas.sizePolicy().hasHeightForWidth())
        self.tw_miniCanvas.setSizePolicy(sizePolicy)
        self.tw_miniCanvas.setTabPosition(QtWidgets.QTabWidget.West)
        self.tw_miniCanvas.setProperty("tabBarAutoHide", False)
        self.tw_miniCanvas.setObjectName("tw_miniCanvas")
        self.tab_3 = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tab_3.sizePolicy().hasHeightForWidth())
        self.tab_3.setSizePolicy(sizePolicy)
        self.tab_3.setObjectName("tab_3")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.tab_3)
        self.verticalLayout_7.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 1)
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.miniCanvasPreview = CanvasPreviewFrame(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.miniCanvasPreview.sizePolicy().hasHeightForWidth())
        self.miniCanvasPreview.setSizePolicy(sizePolicy)
        self.miniCanvasPreview.setMinimumSize(QtCore.QSize(210, 162))
        self.miniCanvasPreview.setAcceptDrops(False)
        self.miniCanvasPreview.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.miniCanvasPreview.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.miniCanvasPreview.setLineWidth(1)
        self.miniCanvasPreview.setObjectName("miniCanvasPreview")
        self.verticalLayout_7.addWidget(self.miniCanvasPreview)
        self.tw_miniCanvas.addTab(self.tab_3, "")
        self.verticalLayout.addWidget(self.splitter)
        self.dockWidget.setWidget(self.dockWidgetContents)
        CarlaHostW.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget)
        self.act_file_new = QtWidgets.QAction(CarlaHostW)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/16x16/document-new.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_new.setIcon(icon8)
        self.act_file_new.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_new.setObjectName("act_file_new")
        self.act_file_open = QtWidgets.QAction(CarlaHostW)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/16x16/document-open.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_open.setIcon(icon9)
        self.act_file_open.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_open.setObjectName("act_file_open")
        self.act_file_save = QtWidgets.QAction(CarlaHostW)
        self.act_file_save.setIcon(icon)
        self.act_file_save.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_save.setObjectName("act_file_save")
        self.act_file_save_as = QtWidgets.QAction(CarlaHostW)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/16x16/document-save-as.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_save_as.setIcon(icon10)
        self.act_file_save_as.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_save_as.setObjectName("act_file_save_as")
        self.act_file_quit = QtWidgets.QAction(CarlaHostW)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/16x16/application-exit.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_quit.setIcon(icon11)
        self.act_file_quit.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_quit.setObjectName("act_file_quit")
        self.act_engine_start = QtWidgets.QAction(CarlaHostW)
        self.act_engine_start.setIcon(icon4)
        self.act_engine_start.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_engine_start.setObjectName("act_engine_start")
        self.act_engine_stop = QtWidgets.QAction(CarlaHostW)
        self.act_engine_stop.setIcon(icon5)
        self.act_engine_stop.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_engine_stop.setObjectName("act_engine_stop")
        self.act_plugin_add = QtWidgets.QAction(CarlaHostW)
        self.act_plugin_add.setIcon(icon2)
        self.act_plugin_add.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugin_add.setObjectName("act_plugin_add")
        self.act_plugin_remove_all = QtWidgets.QAction(CarlaHostW)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/16x16/edit-delete.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_plugin_remove_all.setIcon(icon12)
        self.act_plugin_remove_all.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugin_remove_all.setObjectName("act_plugin_remove_all")
        self.act_plugins_enable = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_enable.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_enable.setObjectName("act_plugins_enable")
        self.act_plugins_disable = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_disable.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_disable.setObjectName("act_plugins_disable")
        self.act_plugins_bypass = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_bypass.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_bypass.setObjectName("act_plugins_bypass")
        self.act_plugins_wet100 = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_wet100.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_wet100.setObjectName("act_plugins_wet100")
        self.act_plugins_mute = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_mute.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_mute.setObjectName("act_plugins_mute")
        self.act_plugins_volume100 = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_volume100.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_volume100.setObjectName("act_plugins_volume100")
        self.act_plugins_center = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_center.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_center.setObjectName("act_plugins_center")
        self.act_transport_play = QtWidgets.QAction(CarlaHostW)
        self.act_transport_play.setCheckable(True)
        self.act_transport_play.setIcon(icon4)
        self.act_transport_play.setObjectName("act_transport_play")
        self.act_transport_stop = QtWidgets.QAction(CarlaHostW)
        self.act_transport_stop.setIcon(icon5)
        self.act_transport_stop.setObjectName("act_transport_stop")
        self.act_transport_backwards = QtWidgets.QAction(CarlaHostW)
        self.act_transport_backwards.setIcon(icon6)
        self.act_transport_backwards.setObjectName("act_transport_backwards")
        self.act_transport_forwards = QtWidgets.QAction(CarlaHostW)
        self.act_transport_forwards.setIcon(icon7)
        self.act_transport_forwards.setObjectName("act_transport_forwards")
        self.act_canvas_arrange = QtWidgets.QAction(CarlaHostW)
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/16x16/view-sort-ascending.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_arrange.setIcon(icon13)
        self.act_canvas_arrange.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_arrange.setObjectName("act_canvas_arrange")
        self.act_canvas_refresh = QtWidgets.QAction(CarlaHostW)
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(":/16x16/view-refresh.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_refresh.setIcon(icon14)
        self.act_canvas_refresh.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_refresh.setObjectName("act_canvas_refresh")
        self.act_canvas_save_image = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_save_image.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_save_image.setObjectName("act_canvas_save_image")
        self.act_canvas_zoom_fit = QtWidgets.QAction(CarlaHostW)
        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap(":/16x16/zoom-fit-best.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_zoom_fit.setIcon(icon15)
        self.act_canvas_zoom_fit.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_zoom_fit.setObjectName("act_canvas_zoom_fit")
        self.act_canvas_zoom_in = QtWidgets.QAction(CarlaHostW)
        icon16 = QtGui.QIcon()
        icon16.addPixmap(QtGui.QPixmap(":/16x16/zoom-in.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_zoom_in.setIcon(icon16)
        self.act_canvas_zoom_in.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_zoom_in.setObjectName("act_canvas_zoom_in")
        self.act_canvas_zoom_out = QtWidgets.QAction(CarlaHostW)
        icon17 = QtGui.QIcon()
        icon17.addPixmap(QtGui.QPixmap(":/16x16/zoom-out.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_zoom_out.setIcon(icon17)
        self.act_canvas_zoom_out.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_zoom_out.setObjectName("act_canvas_zoom_out")
        self.act_canvas_zoom_100 = QtWidgets.QAction(CarlaHostW)
        icon18 = QtGui.QIcon()
        icon18.addPixmap(QtGui.QPixmap(":/16x16/zoom-original.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_canvas_zoom_100.setIcon(icon18)
        self.act_canvas_zoom_100.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_zoom_100.setObjectName("act_canvas_zoom_100")
        self.act_settings_show_toolbar = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_toolbar.setCheckable(True)
        self.act_settings_show_toolbar.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_show_toolbar.setObjectName("act_settings_show_toolbar")
        self.act_settings_configure = QtWidgets.QAction(CarlaHostW)
        icon19 = QtGui.QIcon()
        icon19.addPixmap(QtGui.QPixmap(":/16x16/configure.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_settings_configure.setIcon(icon19)
        self.act_settings_configure.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_configure.setObjectName("act_settings_configure")
        self.act_help_about = QtWidgets.QAction(CarlaHostW)
        self.act_help_about.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_help_about.setObjectName("act_help_about")
        self.act_help_about_juce = QtWidgets.QAction(CarlaHostW)
        self.act_help_about_juce.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_help_about_juce.setObjectName("act_help_about_juce")
        self.act_help_about_qt = QtWidgets.QAction(CarlaHostW)
        self.act_help_about_qt.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_help_about_qt.setObjectName("act_help_about_qt")
        self.act_settings_show_meters = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_meters.setCheckable(True)
        self.act_settings_show_meters.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_show_meters.setObjectName("act_settings_show_meters")
        self.act_settings_show_keyboard = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_keyboard.setCheckable(True)
        self.act_settings_show_keyboard.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_show_keyboard.setObjectName("act_settings_show_keyboard")
        self.act_canvas_show_internal = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_show_internal.setCheckable(True)
        self.act_canvas_show_internal.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_show_internal.setObjectName("act_canvas_show_internal")
        self.act_canvas_show_external = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_show_external.setCheckable(True)
        self.act_canvas_show_external.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_canvas_show_external.setObjectName("act_canvas_show_external")
        self.act_settings_show_time_panel = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_time_panel.setCheckable(True)
        self.act_settings_show_time_panel.setObjectName("act_settings_show_time_panel")
        self.act_settings_show_side_panel = QtWidgets.QAction(CarlaHostW)
        self.act_settings_show_side_panel.setCheckable(True)
        self.act_settings_show_side_panel.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_settings_show_side_panel.setObjectName("act_settings_show_side_panel")
        self.act_file_connect = QtWidgets.QAction(CarlaHostW)
        icon20 = QtGui.QIcon()
        icon20.addPixmap(QtGui.QPixmap(":/16x16/network-connect.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_file_connect.setIcon(icon20)
        self.act_file_connect.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_connect.setObjectName("act_file_connect")
        self.act_file_refresh = QtWidgets.QAction(CarlaHostW)
        self.act_file_refresh.setIcon(icon14)
        self.act_file_refresh.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_file_refresh.setObjectName("act_file_refresh")
        self.act_plugins_compact = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_compact.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_compact.setObjectName("act_plugins_compact")
        self.act_plugins_expand = QtWidgets.QAction(CarlaHostW)
        self.act_plugins_expand.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugins_expand.setObjectName("act_plugins_expand")
        self.act_secret_1 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_1.setObjectName("act_secret_1")
        self.act_secret_2 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_2.setObjectName("act_secret_2")
        self.act_secret_3 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_3.setObjectName("act_secret_3")
        self.act_secret_4 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_4.setObjectName("act_secret_4")
        self.act_secret_5 = QtWidgets.QAction(CarlaHostW)
        self.act_secret_5.setObjectName("act_secret_5")
        self.act_plugin_add_jack = QtWidgets.QAction(CarlaHostW)
        self.act_plugin_add_jack.setIcon(icon2)
        self.act_plugin_add_jack.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_plugin_add_jack.setObjectName("act_plugin_add_jack")
        self.act_engine_config = QtWidgets.QAction(CarlaHostW)
        self.act_engine_config.setIcon(icon19)
        self.act_engine_config.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_engine_config.setObjectName("act_engine_config")
        self.act_engine_panic = QtWidgets.QAction(CarlaHostW)
        icon21 = QtGui.QIcon()
        icon21.addPixmap(QtGui.QPixmap(":/16x16/dialog-warning.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_engine_panic.setIcon(icon21)
        self.act_engine_panic.setMenuRole(QtWidgets.QAction.NoRole)
        self.act_engine_panic.setObjectName("act_engine_panic")
        self.act_engine_panel = QtWidgets.QAction(CarlaHostW)
        self.act_engine_panel.setObjectName("act_engine_panel")
        self.act_canvas_save_image_2x = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_save_image_2x.setObjectName("act_canvas_save_image_2x")
        self.act_canvas_save_image_4x = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_save_image_4x.setObjectName("act_canvas_save_image_4x")
        self.act_canvas_copy_clipboard = QtWidgets.QAction(CarlaHostW)
        self.act_canvas_copy_clipboard.setObjectName("act_canvas_copy_clipboard")
        self.menu_File.addAction(self.act_file_connect)
        self.menu_File.addAction(self.act_file_refresh)
        self.menu_File.addAction(self.act_file_new)
        self.menu_File.addAction(self.act_file_open)
        self.menu_File.addAction(self.act_file_save)
        self.menu_File.addAction(self.act_file_save_as)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.act_file_quit)
        self.menu_Engine.addAction(self.act_engine_start)
        self.menu_Engine.addAction(self.act_engine_stop)
        self.menu_Engine.addAction(self.act_engine_panic)
        self.menu_Engine.addSeparator()
        self.menu_Engine.addAction(self.act_engine_config)
        self.menu_PluginMacros.addAction(self.act_plugins_enable)
        self.menu_PluginMacros.addAction(self.act_plugins_disable)
        self.menu_PluginMacros.addSeparator()
        self.menu_PluginMacros.addAction(self.act_plugins_volume100)
        self.menu_PluginMacros.addAction(self.act_plugins_mute)
        self.menu_PluginMacros.addSeparator()
        self.menu_PluginMacros.addAction(self.act_plugins_wet100)
        self.menu_PluginMacros.addAction(self.act_plugins_bypass)
        self.menu_PluginMacros.addSeparator()
        self.menu_PluginMacros.addAction(self.act_plugins_center)
        self.menu_PluginMacros.addSeparator()
        self.menu_PluginMacros.addAction(self.act_plugins_compact)
        self.menu_PluginMacros.addAction(self.act_plugins_expand)
        self.menu_Plugin.addAction(self.act_plugin_add)
        self.menu_Plugin.addAction(self.act_plugin_add_jack)
        self.menu_Plugin.addAction(self.act_plugin_remove_all)
        self.menu_Plugin.addSeparator()
        self.menu_Plugin.addAction(self.menu_PluginMacros.menuAction())
        self.menu_Canvas_Zoom.addAction(self.act_canvas_zoom_fit)
        self.menu_Canvas_Zoom.addSeparator()
        self.menu_Canvas_Zoom.addAction(self.act_canvas_zoom_in)
        self.menu_Canvas_Zoom.addAction(self.act_canvas_zoom_out)
        self.menu_Canvas_Zoom.addAction(self.act_canvas_zoom_100)
        self.menu_Canvas.addAction(self.act_canvas_show_internal)
        self.menu_Canvas.addAction(self.act_canvas_show_external)
        self.menu_Canvas.addSeparator()
        self.menu_Canvas.addAction(self.act_canvas_arrange)
        self.menu_Canvas.addAction(self.act_canvas_refresh)
        self.menu_Canvas.addAction(self.menu_Canvas_Zoom.menuAction())
        self.menu_Canvas.addSeparator()
        self.menu_Canvas.addAction(self.act_canvas_save_image)
        self.menu_Canvas.addAction(self.act_canvas_save_image_2x)
        self.menu_Canvas.addAction(self.act_canvas_save_image_4x)
        self.menu_Canvas.addAction(self.act_canvas_copy_clipboard)
        self.menu_Settings.addAction(self.act_settings_show_toolbar)
        self.menu_Settings.addAction(self.act_settings_show_meters)
        self.menu_Settings.addAction(self.act_settings_show_keyboard)
        self.menu_Settings.addAction(self.act_settings_show_side_panel)
        self.menu_Settings.addSeparator()
        self.menu_Settings.addAction(self.act_settings_configure)
        self.menu_Help.addAction(self.act_help_about)
        self.menu_Help.addAction(self.act_help_about_juce)
        self.menu_Help.addAction(self.act_help_about_qt)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Engine.menuAction())
        self.menubar.addAction(self.menu_Plugin.menuAction())
        self.menubar.addAction(self.menu_Canvas.menuAction())
        self.menubar.addAction(self.menu_Settings.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())
        self.toolBar.addAction(self.act_file_new)
        self.toolBar.addAction(self.act_file_open)
        self.toolBar.addAction(self.act_file_save)
        self.toolBar.addAction(self.act_file_save_as)
        self.toolBar.addAction(self.act_file_connect)
        self.toolBar.addAction(self.act_file_refresh)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.act_plugin_add)
        self.toolBar.addAction(self.act_plugin_add_jack)
        self.toolBar.addAction(self.act_plugin_remove_all)
        self.toolBar.addAction(self.act_engine_panic)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.act_settings_configure)

        self.retranslateUi(CarlaHostW)
        self.tabWidget.setCurrentIndex(0)
        self.tabUtils.setCurrentIndex(0)
        self.tw_miniCanvas.setCurrentIndex(0)
        self.act_file_quit.triggered.connect(CarlaHostW.close) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(CarlaHostW)

    def retranslateUi(self, CarlaHostW):
        _translate = QtCore.QCoreApplication.translate
        CarlaHostW.setWindowTitle(_translate("CarlaHostW", "MainWindow"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.rack), _translate("CarlaHostW", "Rack"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.patchbay), _translate("CarlaHostW", "Patchbay"))
        self.text_logs.setPlainText(_translate("CarlaHostW", "Loading..."))
        self.logs_save.setText(_translate("CarlaHostW", " Save"))
        self.logs_clear.setText(_translate("CarlaHostW", " Clear"))
        self.logs_clear.setShortcut(_translate("CarlaHostW", "Ctrl+L"))
        self.logs_autoscroll.setText(_translate("CarlaHostW", "Auto-Scroll"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.logs), _translate("CarlaHostW", "Logs"))
        self.label_buffer_size.setText(_translate("CarlaHostW", "Buffer Size:"))
        self.label_sample_rate.setText(_translate("CarlaHostW", "Sample Rate:"))
        self.b_xruns.setText(_translate("CarlaHostW", "? Xruns"))
        self.pb_dsp_load.setFormat(_translate("CarlaHostW", "DSP Load: %p%"))
        self.menu_File.setTitle(_translate("CarlaHostW", "&File"))
        self.menu_Engine.setTitle(_translate("CarlaHostW", "&Engine"))
        self.menu_Plugin.setTitle(_translate("CarlaHostW", "&Plugin"))
        self.menu_PluginMacros.setTitle(_translate("CarlaHostW", "Macros (all plugins)"))
        self.menu_Canvas.setTitle(_translate("CarlaHostW", "&Canvas"))
        self.menu_Canvas_Zoom.setTitle(_translate("CarlaHostW", "Zoom"))
        self.menu_Settings.setTitle(_translate("CarlaHostW", "&Settings"))
        self.menu_Help.setTitle(_translate("CarlaHostW", "&Help"))
        self.toolBar.setWindowTitle(_translate("CarlaHostW", "Tool Bar"))
        self.cb_disk.setItemText(0, _translate("CarlaHostW", "Home"))
        self.tabUtils.setTabText(self.tabUtils.indexOf(self.disk), _translate("CarlaHostW", "Disk"))
        self.group_transport_controls.setTitle(_translate("CarlaHostW", "Playback Controls"))
        self.group_transport_info.setTitle(_translate("CarlaHostW", "Time Information"))
        self.label_transport_frame.setText(_translate("CarlaHostW", "Frame:"))
        self.l_transport_frame.setText(_translate("CarlaHostW", "000\'000\'000"))
        self.label_transport_time.setText(_translate("CarlaHostW", "Time:"))
        self.l_transport_time.setText(_translate("CarlaHostW", "00:00:00"))
        self.label_transport_bbt.setText(_translate("CarlaHostW", "BBT:"))
        self.l_transport_bbt.setText(_translate("CarlaHostW", "000|00|0000"))
        self.group_transport_settings.setTitle(_translate("CarlaHostW", "Settings"))
        self.dsb_transport_bpm.setSuffix(_translate("CarlaHostW", " BPM"))
        self.cb_transport_jack.setText(_translate("CarlaHostW", "Use JACK Transport"))
        self.cb_transport_link.setText(_translate("CarlaHostW", "Use Ableton Link"))
        self.tabUtils.setTabText(self.tabUtils.indexOf(self.w_transport), _translate("CarlaHostW", "Transport"))
        self.act_file_new.setText(_translate("CarlaHostW", "&New"))
        self.act_file_new.setShortcut(_translate("CarlaHostW", "Ctrl+N"))
        self.act_file_open.setText(_translate("CarlaHostW", "&Open..."))
        self.act_file_open.setIconText(_translate("CarlaHostW", "Open..."))
        self.act_file_open.setToolTip(_translate("CarlaHostW", "Open..."))
        self.act_file_open.setShortcut(_translate("CarlaHostW", "Ctrl+O"))
        self.act_file_save.setText(_translate("CarlaHostW", "&Save"))
        self.act_file_save.setShortcut(_translate("CarlaHostW", "Ctrl+S"))
        self.act_file_save_as.setText(_translate("CarlaHostW", "Save &As..."))
        self.act_file_save_as.setIconText(_translate("CarlaHostW", "Save As..."))
        self.act_file_save_as.setToolTip(_translate("CarlaHostW", "Save As..."))
        self.act_file_save_as.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+S"))
        self.act_file_quit.setText(_translate("CarlaHostW", "&Quit"))
        self.act_file_quit.setShortcut(_translate("CarlaHostW", "Ctrl+Q"))
        self.act_engine_start.setText(_translate("CarlaHostW", "&Start"))
        self.act_engine_start.setShortcut(_translate("CarlaHostW", "F5"))
        self.act_engine_stop.setText(_translate("CarlaHostW", "St&op"))
        self.act_engine_stop.setShortcut(_translate("CarlaHostW", "F6"))
        self.act_plugin_add.setText(_translate("CarlaHostW", "&Add Plugin..."))
        self.act_plugin_add.setShortcut(_translate("CarlaHostW", "Ctrl+A"))
        self.act_plugin_remove_all.setText(_translate("CarlaHostW", "&Remove All"))
        self.act_plugins_enable.setText(_translate("CarlaHostW", "Enable"))
        self.act_plugins_disable.setText(_translate("CarlaHostW", "Disable"))
        self.act_plugins_bypass.setText(_translate("CarlaHostW", "0% Wet (Bypass)"))
        self.act_plugins_wet100.setText(_translate("CarlaHostW", "100% Wet"))
        self.act_plugins_mute.setText(_translate("CarlaHostW", "0% Volume (Mute)"))
        self.act_plugins_volume100.setText(_translate("CarlaHostW", "100% Volume"))
        self.act_plugins_center.setText(_translate("CarlaHostW", "Center Balance"))
        self.act_transport_play.setText(_translate("CarlaHostW", "&Play"))
        self.act_transport_play.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+P"))
        self.act_transport_stop.setText(_translate("CarlaHostW", "&Stop"))
        self.act_transport_stop.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+X"))
        self.act_transport_backwards.setText(_translate("CarlaHostW", "&Backwards"))
        self.act_transport_backwards.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+B"))
        self.act_transport_forwards.setText(_translate("CarlaHostW", "&Forwards"))
        self.act_transport_forwards.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+F"))
        self.act_canvas_arrange.setText(_translate("CarlaHostW", "&Arrange"))
        self.act_canvas_arrange.setShortcut(_translate("CarlaHostW", "Ctrl+G"))
        self.act_canvas_refresh.setText(_translate("CarlaHostW", "&Refresh"))
        self.act_canvas_refresh.setShortcut(_translate("CarlaHostW", "Ctrl+R"))
        self.act_canvas_save_image.setText(_translate("CarlaHostW", "Save &Image..."))
        self.act_canvas_zoom_fit.setText(_translate("CarlaHostW", "Auto-Fit"))
        self.act_canvas_zoom_fit.setShortcut(_translate("CarlaHostW", "Home"))
        self.act_canvas_zoom_in.setText(_translate("CarlaHostW", "Zoom In"))
        self.act_canvas_zoom_in.setShortcut(_translate("CarlaHostW", "Ctrl++"))
        self.act_canvas_zoom_out.setText(_translate("CarlaHostW", "Zoom Out"))
        self.act_canvas_zoom_out.setShortcut(_translate("CarlaHostW", "Ctrl+-"))
        self.act_canvas_zoom_100.setText(_translate("CarlaHostW", "Zoom 100%"))
        self.act_canvas_zoom_100.setShortcut(_translate("CarlaHostW", "Ctrl+1"))
        self.act_settings_show_toolbar.setText(_translate("CarlaHostW", "Show &Toolbar"))
        self.act_settings_configure.setText(_translate("CarlaHostW", "&Configure Carla"))
        self.act_help_about.setText(_translate("CarlaHostW", "&About"))
        self.act_help_about_juce.setText(_translate("CarlaHostW", "About &JUCE"))
        self.act_help_about_qt.setText(_translate("CarlaHostW", "About &Qt"))
        self.act_settings_show_meters.setText(_translate("CarlaHostW", "Show Canvas &Meters"))
        self.act_settings_show_keyboard.setText(_translate("CarlaHostW", "Show Canvas &Keyboard"))
        self.act_canvas_show_internal.setText(_translate("CarlaHostW", "Show Internal"))
        self.act_canvas_show_external.setText(_translate("CarlaHostW", "Show External"))
        self.act_settings_show_time_panel.setText(_translate("CarlaHostW", "Show Time Panel"))
        self.act_settings_show_side_panel.setText(_translate("CarlaHostW", "Show &Side Panel"))
        self.act_settings_show_side_panel.setShortcut(_translate("CarlaHostW", "Ctrl+P"))
        self.act_file_connect.setText(_translate("CarlaHostW", "&Connect..."))
        self.act_file_refresh.setText(_translate("CarlaHostW", "&Refresh"))
        self.act_plugins_compact.setText(_translate("CarlaHostW", "Compact Slots"))
        self.act_plugins_expand.setText(_translate("CarlaHostW", "Expand Slots"))
        self.act_secret_1.setText(_translate("CarlaHostW", "Perform secret 1"))
        self.act_secret_2.setText(_translate("CarlaHostW", "Perform secret 2"))
        self.act_secret_3.setText(_translate("CarlaHostW", "Perform secret 3"))
        self.act_secret_4.setText(_translate("CarlaHostW", "Perform secret 4"))
        self.act_secret_5.setText(_translate("CarlaHostW", "Perform secret 5"))
        self.act_plugin_add_jack.setText(_translate("CarlaHostW", "Add &JACK Application..."))
        self.act_engine_config.setText(_translate("CarlaHostW", "&Configure driver..."))
        self.act_engine_panic.setText(_translate("CarlaHostW", "Panic"))
        self.act_engine_panel.setText(_translate("CarlaHostW", "Open custom driver panel..."))
        self.act_canvas_save_image_2x.setText(_translate("CarlaHostW", "Save Image... (2x zoom)"))
        self.act_canvas_save_image_4x.setText(_translate("CarlaHostW", "Save Image... (4x zoom)"))
        self.act_canvas_copy_clipboard.setText(_translate("CarlaHostW", "Copy as Image to Clipboard"))
        self.act_canvas_copy_clipboard.setShortcut(_translate("CarlaHostW", "Ctrl+Shift+C"))
from widgets.canvaspreviewframe import CanvasPreviewFrame
from widgets.digitalpeakmeter import DigitalPeakMeter
from widgets.draggablegraphicsview import DraggableGraphicsView
from widgets.racklistwidget import RackListWidget
import resources_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_refresh.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginRefreshW(object):
    def setupUi(self, PluginRefreshW):
        PluginRefreshW.setObjectName("PluginRefreshW")
        PluginRefreshW.resize(613, 369)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(PluginRefreshW)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.group_types = QtWidgets.QGroupBox(PluginRefreshW)
        self.group_types.setAlignment(QtCore.Qt.AlignCenter)
        self.group_types.setObjectName("group_types")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.group_types)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.ch_ladspa = QtWidgets.QCheckBox(self.group_types)
        self.ch_ladspa.setObjectName("ch_ladspa")
        self.verticalLayout.addWidget(self.ch_ladspa)
        self.ch_dssi = QtWidgets.QCheckBox(self.group_types)
        self.ch_dssi.setObjectName("ch_dssi")
        self.verticalLayout.addWidget(self.ch_dssi)
        self.ch_lv2 = QtWidgets.QCheckBox(self.group_types)
        self.ch_lv2.setObjectName("ch_lv2")
        self.verticalLayout.addWidget(self.ch_lv2)
        self.ch_vst = QtWidgets.QCheckBox(self.group_types)
        self.ch_vst.setObjectName("ch_vst")
        self.verticalLayout.addWidget(self.ch_vst)
        self.ch_vst3 = QtWidgets.QCheckBox(self.group_types)
        self.ch_vst3.setObjectName("ch_vst3")
        self.verticalLayout.addWidget(self.ch_vst3)
        self.ch_au = QtWidgets.QCheckBox(self.group_types)
        self.ch_au.setObjectName("ch_au")
        self.verticalLayout.addWidget(self.ch_au)
        self.ch_jsfx = QtWidgets.QCheckBox(self.group_types)
        self.ch_jsfx.setObjectName("ch_jsfx")
        self.verticalLayout.addWidget(self.ch_jsfx)
        self.line_3 = QtWidgets.QFrame(self.group_types)
        self.line_3.setLineWidth(0)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout.addWidget(self.line_3)
        self.ch_sf2 = QtWidgets.QCheckBox(self.group_types)
        self.ch_sf2.setObjectName("ch_sf2")
        self.verticalLayout.addWidget(self.ch_sf2)
        self.ch_sfz = QtWidgets.QCheckBox(self.group_types)
        self.ch_sfz.setObjectName("ch_sfz")
        self.verticalLayout.addWidget(self.ch_sfz)
        spacerItem1 = QtWidgets.QSpacerItem(40, 5, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout.addItem(spacerItem1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.sep_format = QtWidgets.QFrame(self.group_types)
        self.sep_format.setLineWidth(0)
        self.sep_format.setMidLineWidth(1)
        self.sep_format.setFrameShape(QtWidgets.QFrame.VLine)
        self.sep_format.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.sep_format.setObjectName("sep_format")
        self.horizontalLayout_2.addWidget(self.sep_format)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.ch_native = QtWidgets.QCheckBox(self.group_types)
        self.ch_native.setObjectName("ch_native")
        self.verticalLayout_2.addWidget(self.ch_native)
        self.ch_posix32 = QtWidgets.QCheckBox(self.group_types)
        self.ch_posix32.setObjectName("ch_posix32")
        self.verticalLayout_2.addWidget(self.ch_posix32)
        self.ch_posix64 = QtWidgets.QCheckBox(self.group_types)
        self.ch_posix64.setObjectName("ch_posix64")
        self.verticalLayout_2.addWidget(self.ch_posix64)
        self.ch_win32 = QtWidgets.QCheckBox(self.group_types)
        self.ch_win32.setObjectName("ch_win32")
        self.verticalLayout_2.addWidget(self.ch_win32)
        self.ch_win64 = QtWidgets.QCheckBox(self.group_types)
        self.ch_win64.setObjectName("ch_win64")
        self.verticalLayout_2.addWidget(self.ch_win64)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem3)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.horizontalLayout_3.addWidget(self.group_types)
        spacerItem4 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.group_tools = QtWidgets.QGroupBox(PluginRefreshW)
        self.group_tools.setAlignment(QtCore.Qt.AlignCenter)
        self.group_tools.setFlat(True)
        self.group_tools.setObjectName("group_tools")
        self.gridLayout = QtWidgets.QGridLayout(self.group_tools)
        self.gridLayout.setObjectName("gridLayout")
        self.ico_native = QtWidgets.QLabel(self.group_tools)
        self.ico_native.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_native.setText("")
        self.ico_native.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_native.setScaledContents(True)
        self.ico_native.setObjectName("ico_native")
        self.gridLayout.addWidget(self.ico_native, 0, 0, 1, 1)
        self.ico_posix32 = QtWidgets.QLabel(self.group_tools)
        self.ico_posix32.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_posix32.setText("")
        self.ico_posix32.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_posix32.setScaledContents(True)
        self.ico_posix32.setObjectName("ico_posix32")
        self.gridLayout.addWidget(self.ico_posix32, 1, 0, 1, 1)
        self.label_rdflib = QtWidgets.QLabel(self.group_tools)
        self.label_rdflib.setObjectName("label_rdflib")
        self.gridLayout.addWidget(self.label_rdflib, 5, 1, 1, 1)
        self.label_win64 = QtWidgets.QLabel(self.group_tools)
        self.label_win64.setObjectName("label_win64")
        self.gridLayout.addWidget(self.label_win64, 4, 1, 1, 1)
        self.label_native = QtWidgets.QLabel(self.group_tools)
        self.label_native.setObjectName("label_native")
        self.gridLayout.addWidget(self.label_native, 0, 1, 1, 1)
        self.ico_posix64 = QtWidgets.QLabel(self.group_tools)
        self.ico_posix64.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_posix64.setText("")
        self.ico_posix64.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_posix64.setScaledContents(True)
        self.ico_posix64.setObjectName("ico_posix64")
        self.gridLayout.addWidget(self.ico_posix64, 2, 0, 1, 1)
        self.ico_rdflib = QtWidgets.QLabel(self.group_tools)
        self.ico_rdflib.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_rdflib.setText("")
        self.ico_rdflib.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_rdflib.setScaledContents(True)
        self.ico_rdflib.setObjectName("ico_rdflib")
        self.gridLayout.addWidget(self.ico_rdflib, 5, 0, 1, 1)
        self.label_posix32 = QtWidgets.QLabel(self.group_tools)
        self.label_posix32.setObjectName("label_posix32")
        self.gridLayout.addWidget(self.label_posix32, 1, 1, 1, 1)
        self.ico_win32 = QtWidgets.QLabel(self.group_tools)
        self.ico_win32.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_win32.setText("")
        self.ico_win32.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_win32.setScaledContents(True)
        self.ico_win32.setObjectName("ico_win32")
        self.gridLayout.addWidget(self.ico_win32, 3, 0, 1, 1)
        self.ico_win64 = QtWidgets.QLabel(self.group_tools)
        self.ico_win64.setMaximumSize(QtCore.QSize(16, 16))
        self.ico_win64.setText("")
        self.ico_win64.setPixmap(QtGui.QPixmap(":/16x16/dialog-ok-apply.svgz"))
        self.ico_win64.setScaledContents(True)
        self.ico_win64.setObjectName("ico_win64")
        self.gridLayout.addWidget(self.ico_win64, 4, 0, 1, 1)
        self.label_posix64 = QtWidgets.QLabel(self.group_tools)
        self.label_posix64.setObjectName("label_posix64")
        self.gridLayout.addWidget(self.label_posix64, 2, 1, 1, 1)
        self.label_win32 = QtWidgets.QLabel(self.group_tools)
        self.label_win32.setObjectName("label_win32")
        self.gridLayout.addWidget(self.label_win32, 3, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.group_tools)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem5)
        self.group_options = QtWidgets.QGroupBox(PluginRefreshW)
        self.group_options.setAlignment(QtCore.Qt.AlignCenter)
        self.group_options.setFlat(True)
        self.group_options.setObjectName("group_options")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.group_options)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.ch_do_checks = QtWidgets.QCheckBox(self.group_options)
        self.ch_do_checks.setObjectName("ch_do_checks")
        self.verticalLayout_4.addWidget(self.ch_do_checks)
        self.verticalLayout_3.addWidget(self.group_options)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem6)
        self.horizontalLayout_3.addLayout(self.verticalLayout_3)
        spacerItem7 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem7)
        self.verticalLayout_5.addLayout(self.horizontalLayout_3)
        spacerItem8 = QtWidgets.QSpacerItem(20, 6, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem8)
        self.lv2_restart_notice = QtWidgets.QWidget(PluginRefreshW)
        self.lv2_restart_notice.setObjectName("lv2_restart_notice")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.lv2_restart_notice)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(self.lv2_restart_notice)
        self.label_4.setMaximumSize(QtCore.QSize(22, 22))
        self.label_4.setText("")
        self.label_4.setPixmap(QtGui.QPixmap(":/16x16/dialog-information.svgz"))
        self.label_4.setScaledContents(True)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.label_5 = QtWidgets.QLabel(self.lv2_restart_notice)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_4.addWidget(self.label_5)
        self.verticalLayout_5.addWidget(self.lv2_restart_notice)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.progressBar = QtWidgets.QProgressBar(PluginRefreshW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.progressBar.sizePolicy().hasHeightForWidth())
        self.progressBar.setSizePolicy(sizePolicy)
        self.progressBar.setMaximum(100)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setObjectName("progressBar")
        self.horizontalLayout.addWidget(self.progressBar)
        self.b_start = QtWidgets.QPushButton(PluginRefreshW)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/arrow-right.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_start.setIcon(icon)
        self.b_start.setObjectName("b_start")
        self.horizontalLayout.addWidget(self.b_start)
        self.b_skip = QtWidgets.QPushButton(PluginRefreshW)
        self.b_skip.setObjectName("b_skip")
        self.horizontalLayout.addWidget(self.b_skip)
        self.b_close = QtWidgets.QPushButton(PluginRefreshW)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/16x16/window-close.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_close.setIcon(icon1)
        self.b_close.setObjectName("b_close")
        self.horizontalLayout.addWidget(self.b_close)
        self.verticalLayout_5.addLayout(self.horizontalLayout)

        self.retranslateUi(PluginRefreshW)
        self.b_close.clicked.connect(PluginRefreshW.close) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(PluginRefreshW)

    def retranslateUi(self, PluginRefreshW):
        _translate = QtCore.QCoreApplication.translate
        PluginRefreshW.setWindowTitle(_translate("PluginRefreshW", "Carla - Refresh"))
        self.group_types.setTitle(_translate("PluginRefreshW", "Search for new..."))
        self.ch_ladspa.setText(_translate("PluginRefreshW", "LADSPA"))
        self.ch_dssi.setText(_translate("PluginRefreshW", "DSSI"))
        self.ch_lv2.setText(_translate("PluginRefreshW", "LV2"))
        self.ch_vst.setText(_translate("PluginRefreshW", "VST2"))
        self.ch_vst3.setText(_translate("PluginRefreshW", "VST3"))
        self.ch_au.setText(_translate("PluginRefreshW", "AU"))
        self.ch_jsfx.setText(_translate("PluginRefreshW", "JSFX"))
        self.ch_sf2.setText(_translate("PluginRefreshW", "SF2/3"))
        self.ch_sfz.setText(_translate("PluginRefreshW", "SFZ"))
        self.ch_native.setText(_translate("PluginRefreshW", "Native"))
        self.ch_posix32.setText(_translate("PluginRefreshW", "POSIX 32bit"))
        self.ch_posix64.setText(_translate("PluginRefreshW", "POSIX 64bit"))
        self.ch_win32.setText(_translate("PluginRefreshW", "Windows 32bit"))
        self.ch_win64.setText(_translate("PluginRefreshW", "Windows 64bit"))
        self.group_tools.setTitle(_translate("PluginRefreshW", "Available tools:"))
        self.label_rdflib.setText(_translate("PluginRefreshW", "python3-rdflib (LADSPA-RDF support)"))
        self.label_win64.setText(_translate("PluginRefreshW", "carla-discovery-win64"))
        self.label_native.setText(_translate("PluginRefreshW", "carla-discovery-native"))
        self.label_posix32.setText(_translate("PluginRefreshW", "carla-discovery-posix32"))
        self.label_posix64.setText(_translate("PluginRefreshW", "carla-discovery-posix64"))
        self.label_win32.setText(_translate("PluginRefreshW", "carla-discovery-win32"))
        self.group_options.setTitle(_translate("PluginRefreshW", "Options:"))
        self.ch_do_checks.setToolTip(_translate("PluginRefreshW", "Carla will run small processing checks when scanning the plugins (to make sure they won\'t crash).\n"
"You can disable these checks to get a faster scanning time (at your own risk)."))
        self.ch_do_checks.setText(_translate("PluginRefreshW", "Run processing checks while scanning"))
        self.label_5.setText(_translate("PluginRefreshW", "Restart Carla to list the new LV2 plugins"))
        self.progressBar.setFormat(_translate("PluginRefreshW", "Press \'Scan\' to begin the search"))
        self.b_start.setText(_translate("PluginRefreshW", "Scan"))
        self.b_skip.setText(_translate("PluginRefreshW", ">> Skip"))
        self.b_close.setText(_translate("PluginRefreshW", "Close"))
import resources_rc
